java_import java.util.concurrent.Executors

class PiscinaLogger
  class << self
    attr_accessor :configuration
  end

  def initialize(log_name)
    # Create a buffered pool of size one that will handle writing to the logs
    @thread_pool = Executors.newFixedThreadPool(1)

    # Creates a standard Ruby Logger
    @logger = PiscinaLogger.create_logger(log_name)
  end

  def self.create_logger(log_name)
    log_path = construct_path_to_log(log_name)
    logger = Logger.new(log_path, 'daily')

    logger.formatter = proc do |severity, datetime, progname, msg|
       "#{severity} [#{datetime.strftime('%Y-%m-%d %H:%M:%S.%L')}]: #{msg}\n"
    end

    logger.level = PiscinaLogger.configuration.logging_level

    logger
  end

  def self.construct_path_to_log(log_name)
    raise "No logging directory defined" unless PiscinaLogger.configuration.log_directory

    log_dir = PiscinaLogger.configuration.log_directory
    File.join(log_dir, "#{log_name}_piscina.log")
  end

  def write(message, level)
    @thread_pool.execute do
      @logger.send(level, message)
    end
  end

  [:debug, :info, :warn, :error, :fatal, :unknown].each do |level|
    define_method("#{level.to_s}") do |message|
      write(message, level)
    end
  end

  def shutdown
    # TODO wait for pool to close down before closing logger
    @thread_pool.shutdown
    @logger.close
  end

  # Configure!
  def self.configure
    self.configuration ||= Configuration.new
    yield(configuration)
  end

  class Configuration
    attr_accessor :log_directory
    attr_accessor :logging_level

    def initialize
      @logging_level = Logger::INFO

      # Assume a rails app
      @log_directory = if defined?(Rails)
                         File.join(Rails.root.to_s, "log")
                       else
                         nil
                       end
    end
  end
end
